#include "testApp.h"



//--------------------------------------------------------------
void testApp::setup(){

	ofBackground(255,255,255);

	// 2 output channels,
	// 0 input channels
	// 22050 samples per second
	// 256 samples per buffer
	// 4 num buffers (latency)

	sampleRate 			= 44100;
	phase 				= 0;
	phaseAdder 			= 0.0f;
	phaseAdderTarget 	= 0.0f;
	volume				= 0.1f;
	bNoise 				= false;
	lAudio = new float[256];
	rAudio = new float[256];
	ofSoundStreamSetup(2,0,this, sampleRate,256, 4);

	ofSetFrameRate(60);
	
	
	circlePoint.set(-1, -1);

	capture = false;

}


//--------------------------------------------------------------
void testApp::update(){

}

//--------------------------------------------------------------
void testApp::draw(){

	ofLine(212,400,812,400);

	ofSetColor(0xFF3399);
	ofDrawBitmapString("x sine here",212,395);
//	ofDrawBitmapString("(This is where you'd sign your name with the mouse.\nPretend you can see what you've drawn, even though that makes you a liar.)", 212, 415);
	ofDrawBitmapString("(SOUND JOKES! Not funny, sorry.)", 212, 415);
	
	ofCircle(circlePoint.x, circlePoint.y, 2);

	if( pts.size() > 0 ){
		
		int numPts = pts.size();
		
		output.setColor(0x0088EE);
		output.noFill();
		output.beginShape();
		
		//catmull curves create nice smooth curves between points
		//so actually a lower resolution creates a smoother line
		//and reduce the number of poiints in our eps file.
		int rescaleRes = 6;
		
		for(int i = 0; i < numPts; i++){
			
			//we need to draw the first and last point
			//twice for a catmull curve
			if(i == 0 || i == numPts -1){
				output.curveVertex(pts[i].x, pts[i].y);
			}
			
			if(i % rescaleRes == 0) output.curveVertex(pts[i].x, pts[i].y);
		}
		
		output.endShape();
	}
	
	
	if(capture){
		output.endEPS();
		capture =false;
	}
	
	
}	

//--------------------------------------------------------------
void testApp::keyPressed  (int key){
	if (key == '-'){
		volume -= 0.05;
		volume = MAX(volume, 0);
	} else if (key == '+'){
		volume += 0.05;
		volume = MIN(volume, 1);
	}
}

//--------------------------------------------------------------
void testApp::keyReleased  (int key){

}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
	 
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
	int width = ofGetWidth();
	pan = (float)x / (float)width;
	float height = (float)ofGetHeight();
	float heightPct = ((height-y) / height);
	targetFrequency = 2000.0f * heightPct;
	phaseAdderTarget = (targetFrequency / (float) sampleRate) * TWO_PI;
	
	circlePoint.set(x, y);

	ofHideCursor();
	
	
	pts.push_back(ofPoint());
	int last = pts.size()-1;
	
	pts[last].x = x;
	pts[last].y = y;
	
	
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
	phaseAdderTarget = true;


	pts.push_back(ofPoint());
	pts[0].x = x;
	pts[0].y = y;
	


}


//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){
	phaseAdderTarget = false;
	
	ofShowCursor();
	
	// 7. make sound loop once mouse is released?

}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h){

}
//--------------------------------------------------------------
void testApp::audioRequested 	(float * output, int bufferSize, int nChannels){
	//pan = 0.5f;
	float leftScale = 1 - pan;
	float rightScale = pan;

	// sin (n) seems to have trouble when n is very large, so we
	// keep phase in the range of 0-TWO_PI like this:
	while (phase > TWO_PI){
		phase -= TWO_PI;
	}

	if ( bNoise == true){
		// ---------------------- noise --------------
		for (int i = 0; i < bufferSize; i++){
			lAudio[i] = output[i*nChannels    ] = ofRandomf() * volume * leftScale;
			rAudio[i] = output[i*nChannels + 1] = ofRandomf() * volume * rightScale;
		}
	} else {
		phaseAdder = 0.95f * phaseAdder + 0.05f * phaseAdderTarget;
		for (int i = 0; i < bufferSize; i++){
			phase += phaseAdder;
			float sample = sin(phase);
			lAudio[i] = output[i*nChannels    ] = sample * volume * leftScale;
			rAudio[i] = output[i*nChannels + 1] = sample * volume * rightScale;
		}
	}

}

